// -*- c++ -*-

//
// exporter.h
//
// Copyright 2004 by the authors.  All rights reserved.
//
// Authors: Tero Hasu <tero.hasu@hut.fi>
//

/** To implement this application, we will primarily
    need to use the Client MTM API, so that we can
    retrieve and delete messages. We shall not worry
    about sending the messages anywhere ourselves,
    unless that turns out to be very easy. This is
    just a quick-and-dirty application.

    We are only interested in folders and messages,
    not attachments or services. We will only look
    at Inbox and Sent Items, not Outbox or Drafts.
    We are not picky -- we will store and delete
    _all_ SMS messages; we will touch nothing else,
    though, as we do not have the capability to
    handle anything else.

    // if we wanted to be fancy, we might want
    // to use one of these to provide progress info
    CMsvOperation
*/

#include <charconv.h> // charconv.lib
#include <msvapi.h> // msgs.lib
#include <mtclreg.h> // msgs.lib
#include <smsclnt.h> // smcm.lib
#include "sconfig.hrh"

#ifdef __DO_LOGGING__
#include <flogger.h> // flogger.lib (Symbian 7.0-up)
#endif


#if 0

enum TExporterPanicCodes
{
  EPanicAssertionFailed = 0
};

void EnginePanic(TInt aReason)
{
  _LIT(KPanicCat, "smsexporter");
  User::Panic(KPanicCat, aReason);
}

#endif




class CExporterEngine : public CActive,
  // this observer interface is used to notify
  // us of events that occur in the server
			public MMsvSessionObserver
{
public:
  static CExporterEngine* NewL(RFs& aFs);
  ~CExporterEngine();

  // calls the callback after each handled message
  void ExportMessagesL(const TCallBack& aCallBack);

  // calls the callback after each handled message
  void DeleteMessagesL(const TCallBack& aCallBack);

private: // CActive
  virtual void RunL();
  virtual void DoCancel();
  virtual TInt RunError(TInt aError);
private:
  CExporterEngine(RFs& aFs);
  void ConstructL();
  void ConvertFromUnicode(TDes8& aForeign, const TDesC16& aUnicode) const;
private: // from MMsvSessionObserver
  void HandleSessionEventL(TMsvSessionEvent aEvent,
			   TAny* aArg1,
			   TAny* aArg2,
			   TAny* aArg3);
private:
  // messaging server session handle;
  // we just require one instance of this
  CMsvSession* iMsvSession;

  // used for loading MTMs and obtaining handles to them
  CClientMtmRegistry* iClientMtmRegistry;

  // the MTM we wish to use;
  // we will cast the CBaseMtm to this, more
  // specific type
  CSmsClientMtm* iSmsClientMtm;

  // used to access a persisted message,
  // and to obtain its model;
  // we just require one of these, and
  // can set it to point to another message
  // by setting a different TMsvId
  CMsvEntry* iMsvEntry;

  // the messages to export;
  // a set of TMsvId
public: // making these public so progress can be followed
  CMsvEntrySelection* iList;
  TInt iNextIndex;
  TBool iComplete;
  TBool iDeleting; // false for exporting, true for deleting
private:
  TCallBack iCallBack;

  // for encoding the text file
  CCnvCharacterSetConverter* iCnvCharacterSetConverter;

  // file server session
  RFs& iFs;

  // file to write to
  RFile iFile;

#ifdef __DO_LOGGING__
public:
  // logger, NULL if not available
  RFileLogger *iLogger;
#endif
};
